
/*
 * Aplicació de Comunicacions pel control de la porta d'access a la finca
 */

#include <ESP8266WiFi.h>
#include <WiFiClient.h>
#include <ESP8266WebServer.h>

const char* ssid = "Xarxa";
const char* password = "aWQ";

String resposta;
String oObrir;
String oTancar;
String oAturar;
String pClau="password";

ESP8266WebServer server(80);  //creació del servidor

/* Es tracten les diferents ordres*/
void handleRoot() {
  Serial.println('4');
  mostrarEstat();
}

void aturar() {
  Serial.println('1');
  mostrarEstat();
}

void obrir() {
  Serial.println('2');
  mostrarEstat();
}

void tancar() {
  Serial.println('3');
  mostrarEstat();
}

/* Es llegeix l'estat del serial, li dona format i es retorna al client com a resposta*/ 
void mostrarEstat(){
   String estatRead;
   delay(1000);
   while (Serial.available()>0) {
    estatRead = Serial.readStringUntil('\n');
   }
   
   String resposta="{";
   resposta+="\"estat\":";
   resposta+=estatRead[0] ? estatRead[0] : '-';
   resposta+=",\"pObert\":";
   resposta+=estatRead[1] ? estatRead[1] : '-';
   resposta+=",\"pTancat\":";
   resposta+=estatRead[2] ? estatRead[2] : '-';
   resposta+=",\"ir\":";
   resposta+=estatRead[3] ? estatRead[3] : '-';
   resposta+="}";
   server.send(200, "application/json", resposta);
}

//Xifra l'ordre pasada

String ciOrdre(String str){
  String ordre="";
  char caracter;
  char pasCaracter;
  int tempasc;
  int i = str.length();
  if (pClau.length()>5){
    while (i--) {
      caracter=str.charAt(i);
      pasCaracter=pClau.charAt(i);
      tempasc=caracter +  (caracter-pasCaracter);
      if (tempasc<65){
        tempasc=122-(65-tempasc);
      }else if (tempasc>122){
        tempasc=65+(tempasc-122);
      } 
      ordre+=char(tempasc);
    }
  } else {
  ordre=str;
  }
  ordre.trim();
  return ordre;
}

  
/* Tractament de crides a url que no existeixen*/
void handleNotFound(){
  String message = "Page Not Found\n\n";
  message += "URI: ";
  message += server.uri();
  message += "\nMethod: ";
  message += (server.method() == HTTP_GET)?"GET":"POST";
  message += "\nArguments: ";
  message += server.args();
  message += "\n";
  for (uint8_t i=0; i<server.args(); i++){
    message += " " + server.argName(i) + ": " + server.arg(i) + "\n";
  }
  server.send(404, "text/plain", message);
}

void setup(void){

  Serial.begin(9600); //Configuració de la velocitat a 9600 per no tenir problemes amb la comunicació amb Arduino
  WiFi.begin(ssid, password);
  Serial.println("");
  // Wait for connection
  while (WiFi.status() != WL_CONNECTED) {
    delay(500);
    Serial.print(".");
  }
  Serial.println("");
  Serial.print("Connected to ");
  Serial.println(ssid);
  Serial.print("IP address: ");
  Serial.println(WiFi.localIP());

  // Es xifren les ordres i es guarden xifrades
  oObrir="/" + ciOrdre("obrir");
  oTancar="/" + ciOrdre("tancar");
  oAturar="/" + ciOrdre("aturar");

  server.on("/", handleRoot);
  server.on(oObrir.c_str(), obrir);
  server.on(oTancar.c_str(), tancar);
  server.on(oAturar.c_str(), aturar);

  server.onNotFound(handleNotFound);

  server.begin();
  Serial.println("HTTP server started");
}

void loop(void){
 
  server.handleClient();
}
